@@ -1,16 +1,9 @@
-"""
-    Comparison and analysis functions and wrappers.
-
-    These functions are intended to be used for comparing encodes, filterchains,
-    and other kinds of personal analysis.
-"""
 from __future__ import annotations
 
 import math
 import random
 import warnings
 from abc import ABC, abstractmethod
-from enum import IntEnum
 from itertools import groupby, zip_longest
 from typing import (Any, Callable, Dict, Iterable, Iterator, List, Literal,
                     Sequence, Set, Tuple, TypeVar, overload)
@@ -20,39 +13,43 @@ from vsutil import depth, get_subsampling, get_w
 from vsutil import split as split_planes
 
 from .dehardsub import hardsub_mask
-from .misc import get_matrix
+from .exceptions import (ClipsAndNamedClipsError, InvalidFormatError,
+                         NotEqualFormatsError, VariableFormatError)
+from .kernels import Catrom
 from .render import clip_async_render
-from .util import check_variable, get_prop
+from .types import Direction
+from .util import (check_variable, check_variable_format,
+                   check_variable_resolution, get_matrix, get_prop)
 
 core = vs.core
 
 __all__: List[str] = [
-    'Comparer', 'Stack', 'Interleave', 'Tile', 'Split',
-    'compare', 'comp', 'stack_compare', 'scomp',
+    'compare', 'comp',
+    'Comparer',
+    'diff_hardsub_mask',
+    'diff',
+    'Interleave',
+    'interleave',
+    'Split',
+    'split',
+    'stack_compare', 'scomp',
+    'stack_horizontal',
     'stack_planes',
-    'diff_hardsub_mask', 'diff',
-    'interleave', 'split', 'tile',
-    'stack_horizontal', 'stack_vertical',
-    'Direction'
+    'stack_vertical',
+    'Stack',
+    'Tile',
+    'tile',
 ]
 
 T = TypeVar('T')
 
 
-class Direction(IntEnum):
-    """
-    Enum to simplify direction argument.
-    """
-    HORIZONTAL = 0
-    VERTICAL = 1
-
-
 class Comparer(ABC):
     """
     Base class for comparison functions.
 
     :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlayed on the clips using
+                            If given a dict, the names will be overlaid on the clips using
                             ``VideoNode.text.Text``.
                             If given a simple sequence of clips, the `label_alignment` parameter will have no effect
                             and the clips will not be labeled.
@@ -68,11 +65,11 @@ class Comparer(ABC):
                  label_alignment: int = 7
                  ) -> None:
         if len(clips) < 2:
-            raise ValueError("Comparer: compare functions must be used on at least 2 clips")
+            raise ValueError("Comparer: Compare functions must be used on at least 2 clips!")
         if label_alignment not in list(range(1, 10)):
-            raise ValueError("Comparer: `label_alignment` must be an integer from 1 to 9")
+            raise ValueError("Comparer: `label_alignment` must be an integer from 1 to 9!")
         if not isinstance(clips, (dict, Sequence)):
-            raise ValueError(f"Comparer: unexpected type {type(clips)} for `clips` argument")
+            raise TypeError(f"Comparer: Unexpected type {type(clips)} for `clips` argument!")
 
         self.clips: List[vs.VideoNode] = list(clips.values()) if isinstance(clips, dict) else list(clips)
         self.names: List[str] | None = list(clips.keys()) if isinstance(clips, dict) else None
@@ -92,7 +89,7 @@ class Comparer(ABC):
 
     def _marked_clips(self) -> List[vs.VideoNode]:
         """
-        If a `name` is only space characters, `'   '`, for example, the name will not be overlayed on the clip.
+        If a `name` is only space characters, `'   '`, for example, the name will not be overlaid on the clip.
         """
         if self.names:
             return [clip.text.Text(text=name, alignment=self.label_alignment) if name.strip() else clip
@@ -122,7 +119,7 @@ class Stack(Comparer):
     and either ``vapoursynth.core.std.StackHorizontal`` or ``vapoursynth.core.std.StackVertical``.
 
     :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlayed on the clips using
+                            If given a dict, the names will be overlaid on the clips using
                             ``VideoNode.text.Text``.
                             If given a simple sequence of clips, the `label_alignment` parameter will have no effect
                             and the clips will not be labeled.
@@ -146,11 +143,11 @@ class Stack(Comparer):
     def _compare(self) -> vs.VideoNode:
         if self.direction == Direction.HORIZONTAL:
             if not self.height:
-                raise ValueError("Stack: StackHorizontal requires that all clips be the same height")
+                raise ValueError("Stack: StackHorizontal requires that all clips be the same height!")
             return core.std.StackHorizontal(self._marked_clips())
 
         if not self.width:
-            raise ValueError("Stack: StackVertical requires that all clips be the same width")
+            raise ValueError("Stack: StackVertical requires that all clips be the same width!")
         return core.std.StackVertical(self._marked_clips())
 
 
@@ -163,7 +160,7 @@ class Interleave(Comparer):
     and ``vapoursynth.core.std.Interleave``.
 
     :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlayed on the clips using
+                            If given a dict, the names will be overlaid on the clips using
                             ``VideoNode.text.Text``.
                             If given a simple sequence of clips, the `label_alignment` parameter will have no effect
                             and the clips will not be labeled.
@@ -219,7 +216,7 @@ class Tile(Comparer):
         # the rows will be auto-padded with 0's to be the same length.
 
     :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlayed on the clips using
+                            If given a dict, the names will be overlaid on the clips using
                             ``VideoNode.text.Text``.
                             If given a simple sequence of clips, the `label_alignment` parameter will have no effect
                             and the clips will not be labeled.
@@ -241,14 +238,14 @@ class Tile(Comparer):
         super().__init__(clips, label_alignment=label_alignment)
 
         if not self.width or not self.height:
-            raise ValueError("Tile: all clip widths, and heights must be the same")
+            raise ValueError("Tile: All clip widths and heights must be the same!")
 
         if arrangement is None:
             self.arrangement = self._auto_arrangement()
         else:
             is_one_dim = len(arrangement) < 2 or all(len(row) == 1 for row in arrangement)
             if is_one_dim:
-                raise ValueError("Tile: use Stack instead if the array is one dimensional")
+                raise ValueError("Tile: Use Stack instead if the array is one dimensional!")
             self.arrangement = arrangement
 
         self.blank_clip = core.std.BlankClip(clip=self.clips[0], keep=1)
@@ -256,10 +253,10 @@ class Tile(Comparer):
         max_length = max(map(len, self.arrangement))
         self.arrangement = [row + [0] * (max_length - len(row)) for row in self.arrangement]
 
-        array_count = sum(map(sum, self.arrangement))
+        array_count = sum(map(sum, self.arrangement))  # type:ignore[arg-type]
 
         if array_count != self.num_clips:
-            raise ValueError('Tile: specified arrangement has an invalid number of clips')
+            raise ValueError('Tile: Specified arrangement has an invalid number of clips!')
 
     def _compare(self) -> vs.VideoNode:
         clips = self._marked_clips()
@@ -286,7 +283,7 @@ class Split(Stack):
     For example, five `104 x 200` clips will result in a `((20 x 200) * 4) + (24 x 200)` horiztonal stack of clips.
 
     :param clips:           A dict mapping names to clips or simply a sequence of clips in a tuple or a list.
-                            If given a dict, the names will be overlayed on the clips using
+                            If given a dict, the names will be overlaid on the clips using
                             ``VideoNode.text.Text``.
                             If given a simple sequence of clips, the `label_alignment` parameter will have no effect
                             and the clips will not be labeled.
@@ -311,7 +308,7 @@ class Split(Stack):
     def _smart_crop(self) -> None:  # has to alter self.clips to send clips to _marked_clips() in Stack's _compare()
         """Crops self.clips in place accounting for odd resolutions."""
         if not self.width or not self.height:
-            raise ValueError("Split: all clips must have same width and height")
+            raise ValueError("Split: All clips must have same width and height!")
 
         breaks_subsampling = ((self.direction == Direction.HORIZONTAL and (((self.width // self.num_clips) % 2)
                                                                            or ((self.width % self.num_clips) % 2)))
@@ -321,32 +318,32 @@ class Split(Stack):
         is_subsampled = not all(get_subsampling(clip) in ('444', None) for clip in self.clips)
 
         if breaks_subsampling and is_subsampled:
-            raise ValueError("Split: resulting cropped width or height violates subsampling rules; "
-                             "consider resampling to YUV444 or RGB before attempting to crop")
+            raise ValueError("Split: Resulting cropped width or height violates subsampling rules! "
+                             "Consider resampling to YUV444 or RGB before attempting to crop!")
 
-        if self.direction == Direction.HORIZONTAL:
-            crop_width, overflow = divmod(self.width, self.num_clips)
+        match self.direction:
+            case Direction.HORIZONTAL:
+                crop_width, overflow = divmod(self.width, self.num_clips)
 
-            for key, clip in enumerate(self.clips):
-                left_crop = crop_width * key
-                right_crop = crop_width * (self.num_clips - 1 - key)
+                for key, clip in enumerate(self.clips):
+                    left_crop = crop_width * key
+                    right_crop = crop_width * (self.num_clips - 1 - key)
 
-                if key != (self.num_clips - 1):
-                    right_crop += overflow
+                    if key != (self.num_clips - 1):
+                        right_crop += overflow
 
-                self.clips[key] = clip.std.Crop(left=left_crop, right=right_crop)
+                    self.clips[key] = clip.std.Crop(left=left_crop, right=right_crop)
+            case Direction.VERTICAL:
+                crop_height, overflow = divmod(self.height, self.num_clips)
 
-        elif self.direction == Direction.VERTICAL:
-            crop_height, overflow = divmod(self.height, self.num_clips)
+                for key, clip in enumerate(self.clips):
+                    top_crop = crop_height * key
+                    bottom_crop = crop_height * (self.num_clips - 1 - key)
 
-            for key, clip in enumerate(self.clips):
-                top_crop = crop_height * key
-                bottom_crop = crop_height * (self.num_clips - 1 - key)
+                    if key != (self.num_clips - 1):
+                        bottom_crop += overflow
 
-                if key != (self.num_clips - 1):
-                    bottom_crop += overflow
-
-                self.clips[key] = clip.std.Crop(top=top_crop, bottom=bottom_crop)
+                    self.clips[key] = clip.std.Crop(top=top_crop, bottom=bottom_crop)
 
 
 def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
@@ -377,22 +374,26 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
     def _resample(clip: vs.VideoNode) -> vs.VideoNode:
         # Resampling to 8 bit and RGB to properly display how it appears on your screen
         return core.resize.Bicubic(clip, format=vs.RGB24, matrix_in=get_matrix(clip),
-                                   prefer_props=True, dither_type='error_diffusion')
+                                   dither_type='error_diffusion')
 
-    check_variable(clip_a, "compare")
-    check_variable(clip_b, "compare")
+    check_variable_resolution(clip_a, "compare")
+    check_variable_resolution(clip_b, "compare")
 
     # Error handling
     if frames and len(frames) > clip_a.num_frames:
-        raise ValueError("compare: 'More comparisons requested than frames available'")
+        raise ValueError("compare: 'More comparisons requested than frames available!'")
 
     if force_resample:
         clip_a, clip_b = _resample(clip_a), _resample(clip_b)
     elif mismatch is False:
-        if clip_a.format is None or clip_b.format is None:
-            raise ValueError("compare: 'Variable-format clips not supported'")
+        check_variable_format(clip_a, "compare")
+        check_variable_format(clip_b, "compare")
+
+        assert clip_a.format
+        assert clip_b.format
+
         if clip_a.format.id != clip_b.format.id:
-            raise ValueError("compare: 'The format of both clips must be equal'")
+            raise NotEqualFormatsError("compare")
 
     if print_frame:
         clip_a = clip_a.text.Text("Clip A").text.FrameNum(alignment=9)
@@ -411,9 +412,7 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
 
 def stack_compare(*clips: vs.VideoNode,
                   make_diff: bool = True,
-                  height: int = 288,
-                  warn: Any = None
-                  ) -> vs.VideoNode:
+                  height: int | None = None) -> vs.VideoNode:
     """
     A simple wrapper that allows you to compare two clips by stacking them.
 
@@ -425,27 +424,35 @@ def stack_compare(*clips: vs.VideoNode,
     :param clips:     Clips to compare
     :param make_diff: Create and stack a diff (only works if two clips are given) (Default: ``True``)
     :param height:    Height in px to rescale clips to if `make_diff` is ``True``
-                      (MakeDiff clip will be twice this resolution) (Default: 288)
-    :param warn:      Unused parameter kept for backward compatibility
+                      (MakeDiff clip will be twice this resolution).
+                      This function will not scale above the first clip's height (Default: 288).
 
     :return:          Clip with `clips` stacked
     """
 
     if not make_diff:
         warnings.warn("stack_compare has been deprecated in favour of `lvsfunc.comparison.Stack` "
-                      "when you're not using `make_diff`", DeprecationWarning)
+                      "when not using `make_diff`", DeprecationWarning)
         return core.std.StackHorizontal(clips)
 
     if len(clips) != 2:
         raise ValueError("stack_compare: `make_diff` only works for exactly 2 clips")
 
     clipa, clipb = clips
+
+    if not height:
+        height = 288
+    elif height > clipa.height / 2:
+        warnings.warn(f"stack_compare: 'Given 'height' ({height}) is bigger than clipa's height {clipa.height}!' "
+                      "Will be using clipa's height instead.")
+        height = int(clipa.height / 2)
+
     scaled_width = get_w(height, only_even=False)
 
     diff = core.std.MakeDiff(clipa=clipa, clipb=clipb)
-    diff = diff.resize.Spline36(width=scaled_width * 2, height=height * 2).text.FrameNum(8)
-    resized = [clipa.resize.Spline36(width=scaled_width, height=height).text.Text('Clip A', 3),
-               clipb.resize.Spline36(width=scaled_width, height=height).text.Text('Clip B', 1)]
+    diff = Catrom().scale(diff, scaled_width * 2, height * 2).text.FrameNum(8)
+    resized = [Catrom().scale(clipa, scaled_width, height).text.Text('Clip A', 3),
+               Catrom().scale(clipb, scaled_width, height).text.Text('Clip B', 1)]
 
     return Stack([Stack(resized).clip, diff], direction=Direction.VERTICAL).clip
 
@@ -466,7 +473,7 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
     assert clip.format
 
     if clip.format.num_planes != 3:
-        raise ValueError("stack_planes: input clip must be in YUV or RGB planar format")
+        raise InvalidFormatError("stack_planes", "{func}: Input clip must be of a YUV or RGB planar format!")
 
     yuv_planes = split_planes(clip)
 
@@ -475,7 +482,7 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
     elif clip.format.color_family == vs.ColorFamily.RGB:
         planes = {'R': yuv_planes[0], 'G': yuv_planes[1], 'B': yuv_planes[2]}
     else:
-        raise ValueError(f"stack_planes: unexpected color family {clip.format.color_family.name}")
+        raise TypeError(f"stack_planes: Unexpected color family: {clip.format.color_family.name}!")
 
     direction: Direction = Direction.HORIZONTAL if not stack_vertical else Direction.VERTICAL
 
@@ -490,7 +497,7 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
         return Stack([y_plane, subsampled_planes], direction=direction).clip
 
     else:
-        raise ValueError(f"stack_planes: unexpected subsampling {get_subsampling(clip)}")
+        raise TypeError(f"stack_planes: Unexpected subsampling {get_subsampling(clip)}!")
 
 
 def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
@@ -503,8 +510,8 @@ def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.Vid
 
     :return:  Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`
     """
-    check_variable(a, "stack_planes")
-    check_variable(b, "stack_planes")
+    check_variable(a, "diff_hardsub_mask")
+    check_variable(b, "diff_hardsub_mask")
 
     return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b, **kwargs))
 
@@ -577,18 +584,18 @@ def diff(*clips: vs.VideoNode,
                                 Furthermore, the function will print the ranges of all the diffs found.
     """
     if clips and namedclips:
-        raise ValueError("diff: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("diff")
 
     if (clips and len(clips) != 2) or (namedclips and len(namedclips) != 2):
-        raise ValueError("diff: must pass exactly 2 `clips` or `namedclips`")
+        raise ValueError("diff: Must pass exactly 2 `clips` or `namedclips`!")
 
     if thr >= 128:
-        raise ValueError("diff: `thr` must be below 128")
+        raise ValueError("diff: `thr` must be below 128!")
 
     if clips and any(c.format is None for c in clips):
-        raise ValueError("diff: variable-format clips not supported")
+        raise VariableFormatError("diff")
     elif namedclips and any(nc.format is None for nc in namedclips.values()):
-        raise ValueError("diff: variable-format namedclips not supported")
+        raise VariableFormatError("diff")
 
     def _to_ranges(iterable: List[int]) -> Iterable[Tuple[int, int]]:
         iterable = sorted(set(iterable))
@@ -624,7 +631,7 @@ def diff(*clips: vs.VideoNode,
         clip_async_render(diff, progress="Diffing clips...", callback=_cb)
 
     if not frames:
-        raise ValueError("diff: no differences found")
+        raise StopIteration("diff: No differences found!")
 
     frames.sort()
 
@@ -675,13 +682,13 @@ def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode
     :return: An interleaved clip of all the `clips`/`namedclips` specified
     """
     if clips and namedclips:
-        raise ValueError("interleave: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("interleave")
     return Interleave(clips if clips else namedclips).clip
 
 
 def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
-    Small convenience funciton for splitting clips along the x-axis and then stacking.
+    Small convenience function for splitting clips along the x-axis and then stacking.
     Accounts for odd-resolution clips by giving overflow columns to the last clip specified.
     All clips must have the same dimensions (width and height).
 
@@ -693,7 +700,7 @@ def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
              with all `clips`/`namedclips` represented as individual vertical slices.
     """
     if clips and namedclips:
-        raise ValueError("split: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("split")
     return Split(clips if clips else namedclips, label_alignment=2).clip
 
 
@@ -708,7 +715,7 @@ def stack_horizontal(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Vid
     :return: A horizontal stack of the `clips`/`namedclips`
     """
     if clips and namedclips:
-        raise ValueError("stack_horizontal: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("stack_horizontal")
     return Stack(clips if clips else namedclips).clip
 
 
@@ -723,7 +730,7 @@ def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Video
     :return: A vertical stack of the `clips`/`namedclips`
     """
     if clips and namedclips:
-        raise ValueError("stack_vertical: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("stack_vertical")
     return Stack(clips if clips else namedclips, direction=Direction.VERTICAL).clip
 
 
@@ -742,7 +749,7 @@ def tile(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
              into a rectangular arrrangement
     """
     if clips and namedclips:
-        raise ValueError("tile: positional clips and named keyword clips cannot both be given")
+        raise ClipsAndNamedClipsError("tile")
     return Tile(clips if clips else namedclips).clip
 
 
