@@ -1,6 +1,3 @@
-"""
-    Deblocking functions and wrappers.
-"""
 from __future__ import annotations
 
 from functools import partial
@@ -17,7 +14,7 @@ core = vs.core
 
 
 __all__: List[str] = [
-    'vsdpir', 'autodb_dpir'
+    'autodb_dpir', 'vsdpir'
 ]
 
 
@@ -52,7 +49,7 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
     :param edgevalue:       Remove edges from the edgemask that exceed this threshold (higher means more edges removed)
     :param strs:            A list of DPIR strength values (higher means stronger deblocking).
                             You can pass any arbitrary number of values here.
-                            Sane deblocking strenghts lie between 1–20 for most regular deblocking.
+                            Sane deblocking strengths lie between 1–20 for most regular deblocking.
                             Going higher than 50 is not recommended outside of very extreme cases.
                             The amount of values in strs and thrs need to be equal.
     :param thrs:            A list of thresholds, written as [(EdgeValRef, NextFrameDiff, PrevFrameDiff)].
@@ -146,12 +143,12 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
 
     You must install vs-mlrt. For more information, see the following links:
 
-    * https://github.com/AmusementClub/vs-mlrt
-    * https://github.com/AmusementClub/vs-mlrt/wiki/DPIR
-    * https://github.com/AmusementClub/vs-mlrt/releases/latest
+    * `vs-mlrt <https://github.com/AmusementClub/vs-mlrt>`_
+    * `vs-mlrt DPIR wiki page <https://github.com/AmusementClub/vs-mlrt/wiki/DPIR>`_
+    * `vs-mlrt latest release <https://github.com/AmusementClub/vs-mlrt/releases/latest>`_
 
     Converts to RGB -> runs DPIR -> converts back to original format, and with no subsampling if ``i444=True``.
-    For more information, see https://github.com/cszn/DPIR.
+    For more information, see `the original DPIR repository <https://github.com/cszn/DPIR>`_.
 
     Dependencies:
 
@@ -174,7 +171,7 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
     try:
         from vsmlrt import DPIR, Backend, DPIRModel
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("vsdpir: 'vsmlrt is required to use vsdpir functions.'")
+        raise ModuleNotFoundError("vsdpir: 'missing dependency `vsmlrt`!'")
 
     check_variable(clip, "vsdpir")
     assert clip.format
@@ -187,13 +184,11 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
 
     clip_32 = depth(clip, 32, dither_type=Dither.ERROR_DIFFUSION)
 
-    # TODO: Replace with a match-case?
-    if mode.lower() == 'deblock':
-        model = DPIRModel.drunet_deblocking_color if not is_gray else DPIRModel.drunet_deblocking_grayscale
-    elif mode.lower() == 'denoise':
-        model = DPIRModel.drunet_color if not is_gray else DPIRModel.drunet_gray
-    else:
-        raise ValueError(f"""vsdpir: '"{mode}" is not a valid mode!'""")
+    match mode.lower():
+        case 'deblock': model = DPIRModel.drunet_deblocking_color if not is_gray \
+            else DPIRModel.drunet_deblocking_grayscale
+        case 'denoise': model = DPIRModel.drunet_color if not is_gray else DPIRModel.drunet_gray
+        case _: raise TypeError(f"vsdpir: '\"{mode}\" is not a valid mode!'")
 
     dpir_args |= dict(strength=strength, tiles=tiles, model=model)
 
