@@ -1,6 +1,3 @@
-"""
-    Dehaloing functions and wrappers.
-"""
 from __future__ import annotations
 
 import math
@@ -10,9 +7,9 @@ from typing import Any, Dict, List
 import vapoursynth as vs
 from vsutil import depth, fallback, get_depth, get_y
 
-from .denoise import bm3d
 from .kernels import BSpline, Catrom
 from .mask import maxm, minm
+from .noise import bm3d
 from .types import Shapes
 from .util import (check_variable, clamp_values, force_mod, pick_repair,
                    scale_peak)
@@ -22,8 +19,8 @@ core = vs.core
 
 __all__: List[str] = [
     'bidehalo',
-    'masked_dha',
     'fine_dehalo',
+    'masked_dha',
 ]
 
 
@@ -47,7 +44,7 @@ def bidehalo(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
     :param sigmaR_final:        Bilateral's range weight sigma.
                                 if `None`, same as `sigmaR`
     :param bilateral_args:      Additional parameters to pass to bilateral
-    :param bm3d_args:           Additional parameters to pass to :py:class:`lvsfunc.denoise.bm3d`
+    :param bm3d_args:           Additional parameters to pass to :py:class:`lvsfunc.noise.bm3d`
 
     :return:                    Dehalo'd clip
     """
@@ -104,7 +101,7 @@ def masked_dha(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
                             Must be between 0 and 100.
     :param highsens:        Upper sensitivity range. The higher this is, the more it will process.
                             Must be between 0 and 100.
-    :param rfactor:         Image enlargment factor. Set to >1 to enable some form of aliasing-protection.
+    :param rfactor:         Image enlargement factor. Set to >1 to enable some form of aliasing-protection.
                             Must be greater than 1.
     :param maskpull:        Mask pulling factor
     :param maskpush:        Mask pushing factor
@@ -117,13 +114,13 @@ def masked_dha(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
 
     # Original silently changed values around, which I hate. Throwing errors instead.
     if not all(x >= 1 for x in (rfactor, rx, ry)):
-        raise ValueError("masked_dha: 'rfactor, rx, and ry must all be bigger than 1.0'")
+        raise ValueError("masked_dha: 'rfactor, rx, and ry must all be bigger than 1.0!'")
 
     if not 0 <= darkstr <= 1:
-        raise ValueError("masked_dha: 'darkstr must be between 1.0 and 0.0'")
+        raise ValueError("masked_dha: 'darkstr must be between 1.0 and 0.0!'")
 
     if not all(0 < sens < 100 for sens in (lowsens, highsens)):
-        raise ValueError("masked_dha: 'lowsens and highsens must be between 0 and 100'")
+        raise ValueError("masked_dha: 'lowsens and highsens must be between 0 and 100!'")
 
     # These are the only two I'm going to keep, as these will still give expected results.
     maskpull = clamp_values(maskpull, max_val=254, min_val=0)
@@ -153,7 +150,7 @@ def masked_dha(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
     if ref:
         umfc = get_y(ref)
     else:
-        mmg = core.std.MaskedMerge(clip_ss, clip_y, mask_i)
+        mmg = core.std.MaskedMerge(clip_ss, clip_y, mask_i.std.Limiter())
 
         if rfactor == 1:
             ssc = pick_repair(clip)(clip_y, mmg, 1)
@@ -212,7 +209,7 @@ def fine_dehalo(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
                             Must be between 0 and 100.
     :param highsens:        Upper sensitivity range. The higher this is, the more it will process.
                             Must be between 0 and 100.
-    :param rfactor:         Image enlargment factor. Set to >1 to enable some form of aliasing-protection.
+    :param rfactor:         Image enlargement factor. Set to >1 to enable some form of aliasing-protection.
                             Must be greater than 1.
     :param show_mask:       Return mask clip.
 
@@ -221,7 +218,7 @@ def fine_dehalo(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
     try:
         from havsfunc import DeHalo_alpha
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("decomb: missing dependency 'havsfunc'")
+        raise ModuleNotFoundError("fine_dehalo: missing dependency `havsfunc`!'")
 
     check_variable(clip, "fine_dehalo")
     assert clip.format
@@ -232,13 +229,13 @@ def fine_dehalo(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
 
     # Original silently changed values around, which I hate. Throwing errors instead.
     if not all(x >= 1 for x in (rfactor, rx, ry)):
-        raise ValueError("fine_dehalo: 'rfactor, rx, and ry must all be bigger than 1.0'")
+        raise ValueError("fine_dehalo: 'rfactor, rx, and ry must all be bigger than 1.0!'")
 
     if not 0 <= darkstr <= 1:
-        raise ValueError("fine_dehalo: 'darkstr must be between 1.0 and 0.0'")
+        raise ValueError("fine_dehalo: 'darkstr must be between 1.0 and 0.0!'")
 
     if not all(0 < sens < 100 for sens in (lowsens, highsens)):
-        raise ValueError("fine_dehalo: 'lowsens and highsens must be between 0 and 100'")
+        raise ValueError("fine_dehalo: 'lowsens and highsens must be between 0 and 100!'")
 
     if not 0 <= int(show_mask) < 7:
         raise ValueError("fine_dehalo: 'Valid values for show_mask are 0-7!'")
@@ -277,22 +274,15 @@ def fine_dehalo(clip: vs.VideoNode, ref: vs.VideoNode | None = None,
 
     mask = core.std.Expr([large, shr_med], expr="x y - 2 *")
     mask = core.std.Convolution(mask, matrix=[1] * 9)
-    mask = core.std.Expr([mask], expr="x 2 *")
+    mask = core.std.Expr([mask], expr="x 2 *").std.Limiter()
 
-    # TODO: Match case
-    if show_mask == 1:
-        return mask
-    elif show_mask == 2:
-        return shrink
-    elif show_mask == 3:
-        return edges
-    elif show_mask == 4:
-        return strong
-    elif show_mask == 5:
-        return light
-    elif show_mask == 6:
-        return large
-    elif show_mask == 7:
-        return shr_med
+    match show_mask:
+        case 1: return mask
+        case 2: return shrink
+        case 3: return edges
+        case 4: return strong
+        case 5: return light
+        case 6: return large
+        case 7: return shr_med
 
     return core.std.MaskedMerge(clip, dehaloed, mask, planes=[0])
