@@ -1,6 +1,3 @@
-"""
-    Dehardsubbing functions and wrappers.
-"""
 from __future__ import annotations
 
 from abc import ABC
@@ -16,6 +13,17 @@ from .util import (check_variable, normalize_ranges, replace_ranges,
 
 core = vs.core
 
+__all__: List[str] = [
+    'bounded_dehardsub',
+    'get_all_masks',
+    'hardsub_mask',
+    'HardsubASS',
+    'HardsubLine', 'HardsubLineFade',
+    'HardsubMask',
+    'HardsubSign', 'HardsubSignFade',
+    'HardsubSignKgf',
+]
+
 
 class HardsubMask(DeferredMask, ABC):
     """
@@ -97,7 +105,8 @@ class HardsubSignKgf(HardsubMask):
         try:
             from kagefunc import hardsubmask_fades
         except ModuleNotFoundError:
-            raise ModuleNotFoundError("HardsubSignKgf: missing dependency 'kagefunc'")
+            raise ModuleNotFoundError("HardsubSignKgf: missing dependency `kagefunc`!")
+
         self.hardsubmask_fades = hardsubmask_fades
         self.highpass = highpass
         self.expand = expand
@@ -150,7 +159,8 @@ class HardsubLine(HardsubMask):
         try:
             from kagefunc import hardsubmask
         except ModuleNotFoundError:
-            raise ModuleNotFoundError("HardsubLine: missing dependency 'kagefunc'")
+            raise ModuleNotFoundError("HardsubLine: missing dependency `kagefunc`!")
+
         self.hardsubmask = hardsubmask
         super().__init__(*args, **kwargs)
 
@@ -301,4 +311,4 @@ def hardsub_mask(hrdsb: vs.VideoNode, ref: vs.VideoNode, thresh: float = 0.06,
     hsmf = iterate(hsmf, core.std.Maximum, expand)
     hsmf = iterate(hsmf, core.std.Inflate, inflate)
 
-    return hsmf
+    return hsmf.std.Limiter()
