@@ -1,7 +1,3 @@
-"""
-    Deinterlacing, IVTC, and post-deinterlacing functions and wrappers.
-"""
-
 from __future__ import annotations
 
 import os
@@ -16,26 +12,31 @@ from typing import Any, Dict, List
 import vapoursynth as vs
 from vsutil import Dither, depth, get_depth, get_w, get_y, scale_value
 
-from .comparison import Direction, Stack
+from .comparison import Stack
+from .exceptions import InvalidFramerateError
 from .kernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel
 from .render import get_render_progress
-from .util import (check_variable, force_mod, get_neutral_value, get_prop,
-                   pick_repair)
+from .types import Direction
+from .util import (check_variable, check_variable_format, force_mod,
+                   get_neutral_value, get_prop, pick_repair)
 
 core = vs.core
 
 __all__: List[str] = [
-    'sivtc', 'SIVTC', 'tivtc_vfr', 'TIVTC_VFR',
-    'seek_cycle',
-    'deblend', 'decomb',
+    'deblend',
+    'decomb',
     'descale_fields',
     'fix_telecined_fades',
     'ivtc_credits',
+    'seek_cycle',
+    'sivtc', 'SIVTC',
+    'tivtc_vfr', 'TIVTC_VFR',
     'vinverse'
 ]
 
 main_file = os.path.realpath(sys.argv[0]) if sys.argv[0] else None
 main_file = f"{os.path.splitext(os.path.basename(str(main_file)))[0]}_"
+main_file = "{yourScriptName}_" if main_file in ("__main___", "setup_") else main_file
 
 
 def sivtc(clip: vs.VideoNode, pattern: int = 0,
@@ -131,43 +132,58 @@ def tivtc_vfr(clip: vs.VideoNode,
               tfm_args: Dict[str, Any] = {},
               tdecimate_args: Dict[str, Any] = {}) -> vs.VideoNode:
     """
-    Wrapper for performing TFM and TDecimate on a clip that is supposed to be VFR,
-    including generating a metrics/matches/timecodes txt file.
+    Wrapper for performing TFM and TDecimate on a clip that is supposed to be VFR.
+    Includes automatic generation of a metrics/matches/timecodes txt file.
 
-    Largely based on, if not basically rewritten from, atomchtools.TIVTC_VFR.
+    | This function took *heavy* inspiration from atomchtools.TIVTC_VFR,
+    | and is basically an improved rewrite on the concept.
 
-    Dependencies:
+    .. warning::
+        | When calculating the matches and metrics for the first time, your previewer may error!
+        | To fix this, refresh your previewer! If it still doesn't work, open the ``.ivtc`` directory
+        | and check if the files in there are **0kb**. If they are, **delete them** and run the function again.
+        | You may need to first restart your previewer entirely for it to work!
 
-    * TIVTC
+    Dependencies: TIVTC
 
-    :param clip:                Input clip
-    :param tfmIn:               File location for TFM's matches analysis
-    :param tdecIn:              File location for TDecimate's metrics analysis
-    :param mkvOut:              File location for TDecimate's timecode file output
+    :param clip:                Input clip.
+    :param tfmIn:               File location for TFM's matches analysis.
+                                By default it will be written to ``.ivtc/{yourScriptName}_matches.txt``.
+    :param tdecIn:              File location for TDecimate's metrics analysis.
+                                By default it will be written to ``.ivtc/{yourScriptName}_metrics.txt``.
+    :param timecodes_out:       File location for TDecimate's timecodes analysis.
+                                By default it will be written to ``.ivtc/{yourScriptName}_timecodes.txt``.
     :param decimate:            Perform TDecimate on the clip if true, else returns TFM'd clip only.
-                                Set to -1 to use TDecimate without TFM
-    :param tfm_args:            Additional arguments to pass to TFM
-    :param tdecimate_args:      Additional arguments to pass to TDecimate
+                                Set to -1 to use TDecimate without TFM.
+    :param tfm_args:            Additional arguments to pass to TFM.
+    :param tdecimate_args:      Additional arguments to pass to TDecimate.
 
-    :return:                    IVTC'd VFR clip
+    :return:                    IVTC'd VFR clip with external timecode/matches/metrics txt files
     """
     if int(decimate) not in (-1, 0, 1):
-        raise ValueError("TIVTC_VFR: 'Invalid `decimate` argument. Must be True/False, their integer values, or -1'")
+        raise TypeError("TIVTC_VFR: 'Invalid `decimate` argument. Must be True/False, their integer values, or -1!'")
 
-    tfm_in = Path(tfm_in).resolve()
-    tdec_in = Path(tdec_in).resolve()
-    timecodes_out = Path(timecodes_out).resolve()
+    tfm_f = tdec_f = timecodes_f = Path()
+
+    def _set_paths() -> None:
+        nonlocal tfm_f, tdec_f, timecodes_f
+        tfm_f = Path(tfm_in).resolve().absolute()
+        tdec_f = Path(tdec_in).resolve().absolute()
+        timecodes_f = Path(timecodes_out).resolve().absolute()
+
+    _set_paths()
 
     # TIVTC can't write files into directories that don't exist
-    for p in (tfm_in, tdec_in, timecodes_out):
+    for p in (tfm_f, tdec_f, timecodes_f):
         if not p.parent.exists():
             p.parent.mkdir(parents=True)
 
-    if not (tfm_in.exists() and tdec_in.exists()):
-        tfm_analysis: Dict[str, Any] = {**tfm_args, 'output': str(tfm_in)}
-        tdec_analysis: Dict[str, Any] = {**tdecimate_args, 'output': str(tdec_in), 'mode': 4}
+    if not (tfm_f.exists() and tdec_f.exists()):
+        tfm_analysis: Dict[str, Any] = {**tfm_args, 'output': str(tfm_f)}
+        tdec_analysis: Dict[str, Any] = {'mode': 4, **tdecimate_args, 'output': str(tdec_f)}
 
-        ivtc_clip = core.tivtc.TFM(clip, **tfm_analysis).tivtc.TDecimate(**tdec_analysis)
+        ivtc_clip = core.tivtc.TFM(clip, **tfm_analysis)
+        ivtc_clip = core.tivtc.TDecimate(ivtc_clip, **tdec_analysis)
 
         with get_render_progress() as pr:
             task = pr.add_task("Analyzing frames...", total=ivtc_clip.num_frames)
@@ -178,14 +194,18 @@ def tivtc_vfr(clip: vs.VideoNode,
             with open(os.devnull, 'wb') as dn:
                 ivtc_clip.output(dn, progress_update=_cb)
 
-        time.sleep(0.5)  # Allow it to properly finish writing logs
         del ivtc_clip  # Releases the clip, and in turn the filter (prevents an error)
 
+        _set_paths()
+
+    while not (tfm_f.stat().st_size > 0 and tdec_f.stat().st_size > 0):
+        time.sleep(0.5)  # Allow it to properly finish writing logs if necessary
+
     tfm_args = {**tfm_args, 'input': str(tfm_in)}
 
     tdecimate_args = {
         'mode': 5, 'hybrid': 2, 'vfrDec': 1, **tdecimate_args,
-        'input': str(tdec_in), 'tfmIn': str(tfm_in), 'mkvOut': str(timecodes_out),
+        'input': str(tdec_f), 'tfmIn': str(tfm_f), 'mkvOut': str(timecodes_f),
     }
 
     tfm = clip.tivtc.TFM(**tfm_args) if decimate != -1 else clip
@@ -205,8 +225,8 @@ def deblend(clip: vs.VideoNode, start: int = 0,
     Doing this will result in some of the artifacting being added to the deblended frame,
     but we can mitigate that by repairing the frame with the non-blended frame before it.
 
-    For more information, please refer to this blogpost by torchlight:
-    https://mechaweaponsvidya.wordpress.com/2012/09/13/adventures-in-deblending/
+    For more information, please refer to `this blogpost by torchlight
+    <https://mechaweaponsvidya.wordpress.com/2012/09/13/adventures-in-deblending/>`_.
 
     Dependencies:
 
@@ -278,7 +298,7 @@ def decomb(clip: vs.VideoNode,
     try:
         from havsfunc import QTGMC
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("decomb: missing dependency 'havsfunc'")
+        raise ModuleNotFoundError("decomb: missing dependency `havsfunc`!")
 
     VFM_TFF = int(tff)
 
@@ -296,7 +316,7 @@ def decomb(clip: vs.VideoNode,
     combmask = core.comb.CombMask(clip, cthresh=1, mthresh=3)
     combmask = core.std.Maximum(combmask, threshold=250).std.Maximum(threshold=250) \
         .std.Maximum(threshold=250).std.Maximum(threshold=250)
-    combmask = core.std.BoxBlur(combmask, hradius=2, vradius=2)
+    combmask = core.std.BoxBlur(combmask, hradius=2, vradius=2).std.Limiter()
 
     if show_mask:
         return combmask
@@ -383,8 +403,9 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
     to avoid it damaging frames it shouldn't need to. This helps a lot with orphan fields as well,
     which would otherwise create massive swings in values, sometimes messing up the fade fixing.
 
-    If you pass your own float clip, you'll want to make sure to properly dither it down after.
-    If you don't do this, you'll run into some serious issues!
+    .. warning::
+        | If you pass your own float clip, you'll want to make sure to properly dither it down after.
+        | If you don't do this, you'll run into some serious issues!
 
     Taken from this gist and modified by LightArrowsEXE.
     <https://gist.github.com/blackpilling/bf22846bfaa870a57ad77925c3524eb1>
@@ -412,7 +433,7 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
 
         return core.std.Interleave(fixed).std.DoubleWeave()[::2]
 
-    # I want to catch this before it reaches SeperateFields and give newer users a more useful error
+    # I want to catch this before it reaches separateFields and give newer users a more useful error
     if get_prop(clip.get_frame(0), '_FieldBased', int) == 0 and tff is None:
         raise vs.Error("fix_telecined_fades: 'You must set `tff` for this clip!'")
     elif isinstance(tff, (bool, int)):
@@ -470,7 +491,7 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
     try:
         from havsfunc import QTGMC, DitherLumaRebuild
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("ivtc_credits: missing dependency 'havsfunc'")
+        raise ModuleNotFoundError("ivtc_credits: missing dependency `havsfunc`!")
 
     check_variable(clip, "ivtc_credits")
 
@@ -491,7 +512,7 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
 
         if dec:
             warnings.warn("ivtc_credits: 'Fieldmatched clip passed to function! "
-                          "dec is set to True. If you want to disable this, set dec=False!'")
+                          "dec is set to True. If you want to disable this, set `dec=False`!'")
 
     # motion vector and other values
     field_ref = frame_ref * 2
@@ -512,7 +533,8 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
     bobbed = bob_clip or QTGMC(clip, **qtgmc_kwargs)
 
     if bobbed.fps != Fraction(60000, 1001):
-        raise ValueError("ivtc_credits: 'Your bobbed clip must have a framerate of 60000/1001!'")
+        raise InvalidFramerateError("ivtc_credits", bobbed,
+                                    "{func} 'Your bobbed clip *must* have a framerate of 60000/1001!'")
 
     if interlaced:  # 60i credits. Start of ABBCD
         if dec:  # Decimate the clip instead of properly IVTC
@@ -613,7 +635,7 @@ def vinverse(clip: vs.VideoNode, sstr: float = 2.0,
 
     :return:        Clip with residual combing largely removed
     """
-    check_variable(clip, "vinverse")
+    check_variable_format(clip, "vinverse")
     assert clip.format
 
     if amount > 255:
@@ -621,7 +643,7 @@ def vinverse(clip: vs.VideoNode, sstr: float = 2.0,
 
     neutral = get_neutral_value(clip)
 
-    # Expression to find combing and seperate it from the rest of the clip
+    # Expression to find combing and separate it from the rest of the clip
     find_combs = clip.akarin.Expr(f'{neutral} n! x x 2 * x[0,-1] x[0,1] + + 4 / - n@ +')
 
     # Expression to decomb it (creates blending)
