@@ -1,13 +1,9 @@
-"""
-    Kernels for VapourSynth internal resizers.
-    Intended for use by :py:mod:`lvsfunc.scale` functions.
-"""
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from math import sqrt
 from functools import lru_cache
-from typing import Any, Dict, Generator, Tuple, Sequence, List, Type
+from math import sqrt
+from typing import Any, Dict, Generator, List, Sequence, Tuple, Type
 
 import vapoursynth as vs
 
@@ -15,17 +11,11 @@ core = vs.core
 
 
 __all__: List[str] = [
-    'BSpline', 'Bessel', 'Bicubic', 'BicubicDidee',
-    'BicubicDogWay', 'BicubicSharp', 'Bilinear',
-    'BlackHarris', 'BlackMan', 'BlackManMinLobe',
-    'BlackNuttall', 'Bohman', 'Box', 'Catrom',
-    'Cosine', 'FlatTop', 'FmtConv', 'Gaussian',
-    'Ginseng', 'Hamming', 'Hann', 'Hermite',
-    'Impulse', 'Kaiser', 'Lanczos', 'MinSide',
-    'Mitchell', 'NearestNeighbour', 'Parzen',
-    'Point', 'Quadratic', 'Robidoux', 'RobidouxSharp',
-    'RobidouxSoft', 'Sinc', 'Spline16', 'Spline36',
-    'Spline64', 'Welch', 'Wiener',
+    'Bicubic', 'Bilinear', 'FmtConv', 'Lanczos', 'Point', 'Spline16', 'Spline36', 'Spline64',
+    'Bessel', 'BicubicDidee', 'BicubicDogWay', 'BicubicSharp', 'BlackHarris', 'BlackMan', 'BlackManMinLobe',
+    'BlackNuttall', 'Bohman', 'Box', 'BSpline', 'Catrom', 'Cosine', 'FlatTop', 'Gaussian', 'Ginseng', 'Hamming',
+    'Hann', 'Hermite', 'Impulse', 'Kaiser', 'MinSide', 'Mitchell', 'NearestNeighbour', 'Parzen', 'Point', 'Quadratic',
+    'Robidoux', 'RobidouxSharp', 'RobidouxSoft', 'Sinc', 'Welch', 'Wiener',
     'get_all_kernels', 'get_kernel'
 ]
 
@@ -279,6 +269,77 @@ class Spline64(Kernel):
                                     **self.kwargs)
 
 
+class Example(Kernel):
+    """
+    Example Kernel class for documentation purposes.
+    """
+    def __init__(self, b: float = 0, c: float = 1/2, **kwargs: Any) -> None:
+        self.b = b
+        self.c = c
+        super().__init__(**kwargs)
+
+    def scale(self, clip: vs.VideoNode, width: int, height: int,
+              shift: Tuple[float, float] = (0, 0)) -> vs.VideoNode:
+        """
+        Perform a regular scaling operation
+
+        :param clip:        Input clip
+        :param width:       Output width
+        :param height:      Output height
+        :param shift:       Shift clip during the operation.
+                            Expects a tuple of (src_top, src_left).
+        """
+        return core.resize.Bicubic(clip, width, height,
+                                   src_top=shift[0], src_left=shift[1],
+                                   filter_param_a=self.b, filter_param_b=self.c,
+                                   **self.kwargs)
+
+    def descale(self, clip: vs.VideoNode, width: int, height: int,
+                shift: Tuple[float, float] = (0, 0)) -> vs.VideoNode:
+        """
+        Perform a regular descaling operation
+
+        :param clip:        Input clip
+        :param width:       Output width
+        :param height:      Output height
+        :param shift:       Shift clip during the operation.
+                            Expects a tuple of (src_top, src_left).
+        """
+        return core.descale.Debicubic(clip, width, height, b=self.b,
+                                      c=self.c, src_top=shift[0],
+                                      src_left=shift[1])
+
+    def resample(self, clip: vs.VideoNode, format: vs.PresetFormat | vs.VideoFormat,
+                 matrix: vs.MatrixCoefficients | None = None,
+                 matrix_in: vs.MatrixCoefficients | None = None) -> vs.VideoNode:
+        """
+        Perform a regular resampling operation
+
+        :param clip:        Input clip
+        :param format:      Output format
+        :param matrix:      Output matrix. If `None`, will take the matrix from the input clip's frameprops.
+        :param matrix_in:   Input matrix. If `None`, will take the matrix from the input clip's frameprops.
+
+        :rtype:             ``VideoNode``
+        """
+        return core.resize.Bicubic(clip, format=int(format),
+                                   filter_param_a=self.b, filter_param_b=self.c,
+                                   matrix=matrix, matrix_in=matrix_in,
+                                   **self.kwargs)
+
+    def shift(self, clip: vs.VideoNode, shift: Tuple[float, float] = (0, 0)) -> vs.VideoNode:
+        """
+        Perform a regular shifting operation
+
+        :param clip:        Input clip
+        :param shift:       Shift clip during the operation.
+                            Expects a tuple of (src_top, src_left).
+        """
+        return core.resize.Bicubic(clip, src_top=shift[0], src_left=shift[1],
+                                   filter_param_a=self.b, filter_param_b=self.c,
+                                   **self.kwargs)
+
+
 class BSpline(Bicubic):
     """
     Bicubic b=1, c=0
@@ -351,8 +412,8 @@ class RobidouxSharp(Bicubic):
 
 class BicubicDidee(Bicubic):
     """
-    Kernel inspired by a Didée post.
-    See: https://forum.doom9.org/showthread.php?p=1748922#post1748922.
+    | Kernel inspired by a Didée post.
+    | `See this doom9 post for more information <https://forum.doom9.org/showthread.php?p=1748922#post1748922>`_.
 
     Bicubic b=-0.5, c=0.25
 
@@ -433,7 +494,7 @@ class BlackManMinLobe(FmtConv):
 
 class Sinc(FmtConv):
     """
-    fmtconv's gaussian resizer.
+    fmtconv's sinc resizer.
     """
     kernel = 'sinc'
 
@@ -774,4 +835,4 @@ def get_kernel(name: str) -> Type[Kernel]:
         if kernel.__name__.lower() == search_str:
             return kernel
 
-    raise ValueError(f"get_kernel: 'Unknown kernel: {name}'")
+    raise ValueError(f"get_kernel: 'Unknown kernel: {name}!'")
