@@ -1,6 +1,3 @@
-"""
-    Masking functions and wrappers.
-"""
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
@@ -13,17 +10,20 @@ from vsutil import depth, get_y, iterate, join, split
 
 from . import util
 from .types import Position, Range, Shapes, Size
-from .util import (check_variable, pick_removegrain, replace_ranges,
-                   scale_thresh)
+from .util import (check_variable, check_variable_resolution, pick_removegrain,
+                   replace_ranges, scale_thresh)
 
 core = vs.core
 
 
 __all__: List[str] = [
-    'detail_mask', 'detail_mask_neo',
-    'halo_mask', 'range_mask',
-    'BoundingBox', 'DeferredMask',
-    'mt_xxpand_multi', 'minm', 'maxm'
+    'BoundingBox',
+    'DeferredMask',
+    'detail_mask_neo',
+    'detail_mask',
+    'halo_mask',
+    'mt_xxpand_multi', 'minm', 'maxm',
+    'range_mask',
 ]
 
 
@@ -51,7 +51,7 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
 
     :return:            Detail mask
     """
-    check_variable(clip, "detail_mask")
+    check_variable_resolution(clip, "detail_mask")
 
     brz_a = scale_thresh(brz_a, clip)
     brz_b = scale_thresh(brz_b, clip)
@@ -67,7 +67,7 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
 
     mask = core.std.Expr([mask_a, mask_b], 'x y max')
     mask = util.pick_removegrain(mask)(mask, 22)
-    return util.pick_removegrain(mask)(mask, 11)
+    return util.pick_removegrain(mask)(mask, 11).std.Limiter()
 
 
 def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
@@ -123,7 +123,7 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
     merged = core.std.Expr([blur_pref, prew_mask], "x y +")
     rm_grain = pick_removegrain(merged)(merged, rg_mode)
 
-    return depth(rm_grain, clip.format.bits_per_sample)
+    return depth(rm_grain, clip.format.bits_per_sample).std.Limiter()
 
 
 def halo_mask(clip: vs.VideoNode, rad: int = 2,
@@ -181,7 +181,7 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
     mask = core.std.Expr([expand, excl], expr="x y - 2 *")
     # Additional blurring to amplify the mask
     mask = core.std.Convolution(mask, matrix)
-    return core.std.Expr(mask, expr="x 2 *")
+    return core.std.Expr(mask, expr="x 2 *").std.Limiter()
 
 
 def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
@@ -209,8 +209,7 @@ def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
     if radc == 0:
         clip = get_y(clip)
 
-    if clip.format is None:
-        raise ValueError("range_mask: 'Variable-format clips not supported'")
+    assert clip.format
 
     if clip.format.color_family == vs.GRAY:
         ma = _minmax(clip, rad, True)
@@ -224,15 +223,20 @@ def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
             planes[i] = core.std.Expr([ma, mi], 'x y -')
         mask = join(planes)
 
-    return mask
+    return mask.std.Limiter()
 
 
 # Helper functions
 def _minmax(clip: vs.VideoNode, iterations: int, maximum: bool) -> vs.VideoNode:
+    check_variable(clip, "_minmax")
+    assert clip.format
+
     func = core.std.Maximum if maximum else core.std.Minimum
+
     for i in range(1, iterations + 1):
         coord = [0, 1, 0, 1, 1, 0, 1, 0] if (i % 3) != 1 else [1] * 8
         clip = func(clip, coordinates=coord)
+
     return clip
 
 
@@ -264,9 +268,7 @@ class BoundingBox():
         :return:    Square mask representing the bounding box.
         """
         check_variable(ref, "get_mask")
-
-        if ref.format is None:
-            raise ValueError("BoundingBox: 'Variable-format clips not supported'")
+        assert ref.format
 
         if self.pos.x + self.size.x > ref.width or self.pos.y + self.size.y > ref.height:
             raise ValueError("BoundingBox: Bounds exceed clip size!")
@@ -279,7 +281,7 @@ class BoundingBox():
         mask = mask.std.AddBorders(self.pos.x, ref.width - (self.pos.x + self.size.x),
                                    self.pos.y, ref.height - (self.pos.y + self.size.y))
 
-        return mask
+        return mask.std.Limiter()
 
 
 class DeferredMask(ABC):
@@ -360,6 +362,8 @@ class DeferredMask(ABC):
                              range=CRange.FULL, range_in=CRange.FULL)
                 hm = replace_ranges(hm, core.std.Expr([hm, mask*len(hm)], expr="x y max"), range)
 
+        hm = hm.std.Limiter()
+
         return hm if self.bound is None else core.std.MaskedMerge(core.std.BlankClip(hm), hm, bm)
 
     @abstractmethod
@@ -387,14 +391,11 @@ def mt_xxpand_multi(clip: vs.VideoNode,
 
     sh = sh or sw
 
-    # TODO: Match case
-    if mode == Shapes.ELLIPSE:
-        coordinates = [[1] * 8, [0, 1, 0, 1, 1, 0, 1, 0],
-                       [0, 1, 0, 1, 1, 0, 1, 0]]
-    elif mode == Shapes.LOSANGE:
-        coordinates = [[0, 1, 0, 1, 1, 0, 1, 0]] * 3
-    else:
-        coordinates = [[1] * 8] * 3
+    match mode:
+        case Shapes.ELLIPSE: coordinates = [[1] * 8, [0, 1, 0, 1, 1, 0, 1, 0],
+                                                     [0, 1, 0, 1, 1, 0, 1, 0]]
+        case Shapes.LOSANGE: coordinates = [[0, 1, 0, 1, 1, 0, 1, 0]] * 3
+        case _: coordinates = [[1] * 8] * 3
 
     clips = [clip]
     end = int(min(sw, sh)) + start
@@ -406,7 +407,7 @@ def mt_xxpand_multi(clip: vs.VideoNode,
         clips += [m__imum(clips[-1], coordinates=[0, 0, 0, 1, 1, 0, 0, 0], **params)]
 
     for x in range(end, int(end + sh - sw)):
-        clips += [m__imum(clips[-1], coordinates=[0, 1, 0, 0, 0, 0, 1, 0], **params)]
+        clips += [m__imum(clips[-1], coordinates=[0, 1, 0, 0, 0, 0, 1, 0], **params).std.Limiter()]
 
     return clips
 
