@@ -1,40 +1,31 @@
-"""
-    Miscellaneous functions and wrappers that don't really have a place elsewhere.
-"""
 from __future__ import annotations
 
-import colorsys
 import os
-import random
 import warnings
-from functools import partial, wraps
-from typing import Any, Callable, Dict, List, Sequence, Tuple, TypeVar, cast
+from functools import partial
+from typing import Any, List, Sequence, Tuple
 
 import vapoursynth as vs
-from vsutil import depth, get_depth, get_w, get_y, is_image, scale_value
+from vsutil import depth, get_depth, is_image, scale_value
 
+from .exceptions import InvalidMatrixError
 from .kernels import Catrom
 from .mask import BoundingBox
 from .types import Matrix, Position, Range, Size
-from .util import check_variable, get_prop, normalize_ranges
-from .util import replace_ranges as _replace_ranges
-from .util import scale_thresh as _scale_thresh
+from .util import (check_variable, get_matrix, get_prop, normalize_ranges,
+                   replace_ranges)
 
 core = vs.core
 
 
 __all__: List[str] = [
-    'source', 'src',
     'edgefixer', 'ef',
-    'get_matrix',
-    'shift_tint',
     'limit_dark',
-    'wipe_row',
-    'load_bookmarks', 'frames_since_bookmark',
-    'allow_variable', 'chroma_injector',
-    'colored_clips',
+    'overlay_sign',
+    'shift_tint',
+    'source', 'src',
     'unsharpen',
-    'overlay_sign'
+    'wipe_row',
 ]
 
 
@@ -54,8 +45,9 @@ def source(file: str, ref: vs.VideoNode | None = None,
 
     Alias for this function is `lvsfunc.src`.
 
-    WARNING: This function may be rewritten in the future, and functionality may change!
-             No warning is currently printed for this in your terminal to avoid spam.
+    .. warning::
+        | WARNING: This function will be rewritten in the future, and functionality may change!
+        |         No warning is currently printed for this in your terminal to avoid spam.
 
     Dependencies:
 
@@ -65,49 +57,48 @@ def source(file: str, ref: vs.VideoNode | None = None,
     * dgdecodenv (optional: dgi sources)
     * VapourSynth-ReadMpls (optional: mpls sources)
 
-    :param file:              Input file
+    :param file:              Input file. This MUST have an extension.
     :param ref:               Use another clip as reference for the clip's format,
-                              resolution, and framerate (Default: None)
+                              resolution, and framerate (Default: None).
     :param force_lsmas:       Force files to be imported with L-SMASH (Default: False)
-    :param mpls:              Load in a mpls file (Default: False)
-    :param mpls_playlist:     Playlist number, which is the number in mpls file name (Default: 0)
-    :param mpls_angle:        Angle number to select in the mpls playlist (Default: 0)
-    :param kwargs:            Arguments passed to the indexing filter
+    :param mpls:              Load in a mpls file (Default: False).
+    :param mpls_playlist:     Playlist number, which is the number in mpls file name (Default: 0).
+    :param mpls_angle:        Angle number to select in the mpls playlist (Default: 0).
+    :param kwargs:            Arguments passed to the indexing filter.
 
-    :return:                  Vapoursynth clip representing input file
+    :return:                  Vapoursynth clip representing input file.
     """
-    # TODO: find a way to NOT have to rely on a million elif's
     if file.startswith('file:///'):
         file = file[8::]
 
+    fname, ext = os.path.splitext(file)
+
+    if not ext:
+        raise ValueError("source: 'No extension found in filename!'")
+
     # Error handling for some file types
-    if file.endswith('.mpls') and mpls is False:
+    if ext == '.mpls' and mpls is False:
         raise ValueError("source: 'Set \"mpls = True\" and pass a path to the base Blu-ray directory "
-                         "for this kind of file'")
-    if os.path.splitext(file)[1].lower() in annoying_formats:
-        raise ValueError("source: 'Use an external indexer like d2vwitch or DGIndexNV for this kind of file'")
+                         "for this kind of file!'")
+
+    if ext in annoying_formats:
+        raise ValueError("source: 'Use an external indexer like d2vwitch or DGIndexNV for this kind of file!'")
 
     if force_lsmas:
         clip = core.lsmas.LWLibavSource(file, **index_args)
-
     elif mpls:
         mpls_in = core.mpls.Read(file, mpls_playlist, mpls_angle)
-        clip = core.std.Splice([core.lsmas.LWLibavSource(mpls_in['clip'][i], **index_args)
-                                for i in range(mpls_in['count'])])
-
-    elif file.endswith('.d2v'):
-        clip = core.d2v.Source(file, **index_args)
-    elif file.endswith('.dgi'):
-        clip = core.dgdecodenv.DGSource(file, **index_args)
-    elif file.endswith('.mp4'):
-        clip = core.lsmas.LibavSMASHSource(file, **index_args)
+        clip = core.std.Splice([core.lsmas.LWLibavSource(mpls_in['clip'][i], **index_args)  # type:ignore[call-overload]
+                                for i in range(mpls_in['count'])])  # type:ignore[call-overload]
     elif is_image(file):
         clip = core.imwri.Read(file, **index_args)
     else:
-        if file.endswith('.m2ts'):
-            clip = core.lsmas.LWLibavSource(file, **index_args)
-        else:
-            clip = core.ffms2.Source(file, **index_args)
+        match ext:
+            case 'd2v': clip = core.d2v.Source(file, **index_args)
+            case '.dgi': clip = core.dgdecodenv.DGSource(file, **index_args)
+            case '.mp4': clip = core.lsmas.LibavSMASHSource(file, **index_args)
+            case '.m2ts': clip = core.lsmas.LWLibavSource(file, **index_args)
+            case _: clip = core.ffms2.Source(file, **index_args)
 
     if ref:
         check_variable(ref, "source")
@@ -139,7 +130,8 @@ def edgefixer(clip: vs.VideoNode,
 
     Alias for this function is `lvsfunc.ef`.
 
-    WARNING: This function may be rewritten in the future, and functionality may change!
+    .. warning::
+        This function may be rewritten in the future, and functionality may change!
 
     Dependencies:
 
@@ -170,31 +162,9 @@ def edgefixer(clip: vs.VideoNode,
     if bottom is None:
         bottom = top
 
-    ef = core.edgefixer.ContinuityFixer(clip, left, top, right, bottom, radius)
-    return ef if full_range else core.std.Limiter(ef, 16.0, [235, 240])
-
-
-def get_matrix(clip: vs.VideoNode) -> int:
-    """
-    Helper function to get the matrix for a clip.
-
-    :param clip:    src clip
-
-    :return:        Value representing a matrix
-    """
-    check_variable(clip, "get_matrix")
-    assert clip.format
-
-    frame = clip.get_frame(0)
-    w, h = frame.width, frame.height
-
-    if frame.format.color_family == vs.RGB:
-        return 0
-    if w <= 1024 and h <= 576:
-        return 5
-    if w <= 2048 and h <= 1536:
-        return 1
-    return 9
+    ef = core.cf.ContinuityFixer(clip, left, top, right, bottom, radius)
+    limit: vs.VideoNode = ef if full_range else core.std.Limiter(ef, 16.0, [235, 240])
+    return limit
 
 
 def shift_tint(clip: vs.VideoNode, values: int | Sequence[int] = 16) -> vs.VideoNode:
@@ -227,10 +197,10 @@ def shift_tint(clip: vs.VideoNode, values: int | Sequence[int] = 16) -> vs.Video
     elif len(values) == 3:
         val = (values[0], values[1], values[2])
     else:
-        raise ValueError("shift_tint: 'Too many values supplied'")
+        raise ValueError("shift_tint: 'Too many values supplied!'")
 
     if any(v > 255 or v < -255 for v in val):
-        raise ValueError("shift_tint: 'Every value in \"values\" must be below 255'")
+        raise ValueError("shift_tint: 'Every value in \"values\" must be below 255!'")
 
     cdepth = get_depth(clip)
     cv: List[float] = [scale_value(v, 8, cdepth) for v in val] if cdepth != 8 else list(val)
@@ -265,7 +235,7 @@ def limit_dark(clip: vs.VideoNode, filtered: vs.VideoNode,
 
     if threshold_range and threshold_range > threshold:
         raise ValueError(f"limit_dark: '\"threshold_range\" ({threshold_range}) must be "
-                         "a lower value than \"threshold\" ({threshold})'")
+                         f"a lower value than \"threshold\" ({threshold})!'")
 
     avg = core.std.PlaneStats(clip)
     return core.std.FrameEval(clip, partial(_diff, clip=clip, filtered=filtered,
@@ -304,214 +274,6 @@ def wipe_row(clip: vs.VideoNode,
     return core.std.MaskedMerge(clip, ref, sqmask)
 
 
-def load_bookmarks(bookmark_path: str) -> List[int]:
-    """
-    VSEdit bookmark loader.
-
-    load_bookmarks(os.path.basename(__file__)+".bookmarks")
-    will load the VSEdit bookmarks for the current Vapoursynth script.
-
-    :param bookmark_path:  Path to bookmarks file
-
-    :return:               A list of bookmarked frames
-    """
-    with open(bookmark_path) as f:
-        bookmarks = [int(i) for i in f.read().split(", ")]
-
-        if bookmarks[0] != 0:
-            bookmarks.insert(0, 0)
-
-    return bookmarks
-
-
-def frames_since_bookmark(clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
-    """
-    Displays frames since last bookmark to create easily reusable scenefiltering.
-    Can be used in tandem with :py:func:`lvsfunc.misc.load_bookmarks` to import VSEdit bookmarks.
-
-    :param clip:        Input clip
-    :param bookmarks:   A list of bookmarks
-
-    :return:            Clip with bookmarked frames
-    """
-    def _frames_since_bookmark(n: int, clip: vs.VideoNode, bookmarks: List[int]) -> vs.VideoNode:
-        for i, bookmark in enumerate(bookmarks):
-            frames_since = n - bookmark
-
-            if frames_since >= 0 and i + 1 >= len(bookmarks):
-                result = frames_since
-            elif frames_since >= 0 and n - bookmarks[i + 1] < 0:
-                result = frames_since
-                break
-
-        return core.text.Text(clip, str(result))
-    return core.std.FrameEval(clip, partial(_frames_since_bookmark, clip=clip, bookmarks=bookmarks))
-
-
-F = TypeVar("F", bound=Callable[..., vs.VideoNode])
-
-
-def allow_variable(width: int | None = None, height: int | None = None,
-                   format: int | None = None
-                   ) -> Callable[[Callable[..., vs.VideoNode]], Callable[..., vs.VideoNode]]:
-    """
-    Decorator allowing a variable-res and/or variable-format clip to be passed
-    to a function that otherwise would not be able to accept it. Implemented by
-    FrameEvaling and resizing the clip to each frame. Does not work when the
-    function needs to return a different format unless an output format is
-    specified. As such, this decorator must be called as a function when used
-    (e.g. ``@allow_variable()`` or ``@allow_variable(format=vs.GRAY16)``). If
-    the provided clip is variable format, no output format is required to be
-    specified.
-
-    :param width:       Output clip width
-    :param height:      Output clip height
-    :param format:      Output clip format
-
-    :return:            Function decorator for the given output format.
-    """
-    if height is not None:
-        width = width if width else get_w(height)
-
-    def inner(func: F) -> F:
-        @wraps(func)
-        def inner2(clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
-            def frameeval_wrapper(n: int, f: vs.VideoFrame) -> vs.VideoNode:
-                res = func(clip.resize.Point(f.width, f.height, format=f.format.id), *args, **kwargs)
-                return res.resize.Point(format=format) if format else res
-
-            clip_out = clip.resize.Point(format=format) if format else clip
-            clip_out = clip_out.resize.Point(width, height) if width and height else clip_out
-            return core.std.FrameEval(clip_out, frameeval_wrapper, prop_src=[clip])
-
-        return cast(F, inner2)
-
-    return inner
-
-
-def chroma_injector(func: F) -> F:
-    """
-    Decorator allowing injection of reference chroma into a function which
-    would normally only receive luma, such as an upscaler passed to
-    :py:func:`lvsfunc.scale.descale`. The chroma is resampled to the input
-    clip's width, height, and pixel format, shuffled to YUV444PX, then passed
-    to the function. Luma is then extracted from the function result and
-    returned. The first argument of the function is assumed to be the luma
-    source. This works with variable resolution and may work with variable
-    format, however the latter is wholly untested and likely a bad idea in
-    every conceivable use case.
-
-    :param func:        Function to call with injected chroma
-
-    :return:            Decorated function
-    """
-
-    @wraps(func)
-    def inner(_chroma: vs.VideoNode, clip: vs.VideoNode, *args: Any,
-              **kwargs: Any) -> vs.VideoNode:
-
-        def upscale_chroma(n: int, f: vs.VideoFrame) -> vs.VideoNode:
-            luma = y.resize.Point(f.width, f.height, format=f.format.id)
-            if out_fmt is not None:
-                fmt = out_fmt
-            else:
-                fmt = core.register_format(vs.YUV, f.format.sample_type,
-                                           f.format.bits_per_sample, 0, 0)
-            chroma = _chroma.resize.Spline36(f.width, f.height,
-                                             format=fmt.id)
-            res = core.std.ShufflePlanes([luma, chroma], planes=[0, 1, 2],
-                                         colorfamily=vs.YUV)
-            return res
-
-        out_fmt: vs.VideoFormat | None = None
-        if clip.format is not None:
-            if clip.format.color_family not in (vs.GRAY, vs.YUV):
-                raise ValueError("chroma_injector: only YUV and GRAY clips are supported")
-
-            in_fmt = core.register_format(vs.GRAY, clip.format.sample_type,
-                                          clip.format.bits_per_sample, 0, 0)
-            y = allow_variable(format=in_fmt.id)(get_y)(clip)
-            # We want to use YUV444PX for chroma injection
-            out_fmt = core.register_format(vs.YUV, clip.format.sample_type,
-                                           clip.format.bits_per_sample, 0, 0)
-        else:
-            y = allow_variable()(get_y)(clip)
-
-        if y.width != 0 and y.height != 0 and out_fmt is not None:
-            chroma = _chroma.resize.Spline36(y.width, y.height, format=out_fmt.id)
-            clip_in = core.std.ShufflePlanes([y, chroma], planes=[0, 1, 2],
-                                             colorfamily=vs.YUV)
-        else:
-            y_f = y.resize.Point(format=out_fmt.id) if out_fmt is not None else y
-            clip_in = core.std.FrameEval(y_f, upscale_chroma, prop_src=[y])
-
-        result = func(clip_in, *args, **kwargs)
-
-        if result.format is not None:
-            if result.format.color_family not in (vs.GRAY, vs.YUV):
-                raise ValueError("chroma_injector: can only decorate function with YUV and/or GRAY format return")
-
-            if result.format.color_family == vs.GRAY:
-                return result
-
-            res_fmt = core.register_format(vs.GRAY, result.format.sample_type,
-                                           result.format.bits_per_sample, 0, 0)
-            return allow_variable(format=res_fmt.id)(get_y)(result)
-        else:
-            return allow_variable()(get_y)(result)
-
-    return cast(F, inner)
-
-
-def colored_clips(amount: int,
-                  max_hue: int = 300,
-                  rand: bool = True,
-                  seed: bytearray | bytes | float | str | None = None,
-                  **kwargs: Any
-                  ) -> List[vs.VideoNode]:
-    """
-    Returns a list of BlankClips with unique colors in sequential or random order.
-    The colors will be evenly spaced by hue in the HSL colorspace.
-
-    Useful maybe for comparison functions or just for getting multiple uniquely colored BlankClips for testing purposes.
-
-    Will always return a pure red clip in the list as this is the RGB equivalent of the lowest HSL hue possible (0).
-
-    Written by Dave <orangechannel@pm.me>.
-
-    :param amount:  Number of ``vapoursynth.VideoNode``\\s to return
-    :param max_hue: Maximum hue (0 < hue <= 360) in degrees to generate colors from (uses the HSL color model).
-                    Setting this higher than ``315`` will result in the clip colors looping back towards red
-                    and is not recommended for visually distinct colors.
-                    If the `amount` of clips is higher than the `max_hue` expect there to be identical
-                    or visually similar colored clips returned (Default: 300)
-    :param rand:    Randomizes order of the returned list (Default: True)
-    :param seed:    Bytes-like object passed to ``random.seed`` which allows for consistent randomized order
-                    of the resulting clips (Default: None)
-    :param kwargs:  Arguments passed to ``vapoursynth.core.std.BlankClip`` (Default: keep=1)
-
-    :return:        List of uniquely colored clips in sequential or random order.
-    """
-    if amount < 2:
-        raise ValueError("colored_clips: `amount` must be at least 2")
-    if not (0 < max_hue <= 360):
-        raise ValueError("colored_clips: `max_hue` must be greater than 0 and less than 360 degrees")
-
-    blank_clip_args: Dict[str, Any] = {'keep': 1, **kwargs}
-
-    hues: List[float] = [i * max_hue / (amount - 1) for i in range(amount - 1)]
-    hues.append(max_hue)
-
-    hls_color_list: List[Tuple[float, float, float]] = [colorsys.hls_to_rgb(h / 360, 0.5, 1) for h in hues]
-    rgb_color_list = [[int(f * 255) for f in color] for color in hls_color_list]
-
-    if rand:
-        shuffle = random.shuffle if seed is None else random.Random(seed).shuffle
-        shuffle(rgb_color_list)
-
-    return [core.std.BlankClip(color=color, **blank_clip_args) for color in rgb_color_list]
-
-
 def unsharpen(clip: vs.VideoNode, strength: float = 1.0, sigma: float = 1.5,
               prefilter: bool = True, prefilter_sigma: float = 0.75) -> vs.VideoNode:
     """
@@ -557,13 +319,17 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
     Wrapper to overlay a logo or sign onto another clip. Rewrite of fvsfunc.InsertSign.
     This wrapper also allows you to set fades to fade a logo in and out.
 
-    Requires:
+    Dependencies:
 
     * vs-imwri
 
-    :param clip:            Base clip
+    Optional Dependencies:
+
+    * kagefunc
+
+    :param clip:            Input clip.
     :param overlay:         Sign or logo to overlay. Must be the png loaded in through imwri.Read()
-                            or a path string to the image file.
+                            or a path string to the image file, and MUST be the same dimensions as the input clip.
     :param frame_ranges:    Frame ranges or starting frame to apply the overlay to. See ``types.Range`` for more info.
                             If None, overlays the entire clip.
                             If a Range is passed, the overlaid clip will only show up inside that range.
@@ -577,13 +343,14 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
                             If not specified, gets matrix from the "_Matrix" prop of the clip unless it's an RGB clip,
                             in which case it stays as `None`.
 
-    :return:                Clip with a logo or sign overlaid ontop for the given frame ranges,
-                            either with or without a fade
+    :return:                Clip with a logo or sign overlaid on top for the given frame ranges,
+                            either with or without a fade.
     """
-    try:
-        from kagefunc import crossfade
-    except ModuleNotFoundError:
-        raise ModuleNotFoundError("overlay_sign: 'missing dependency `kagefunc`'")
+    if fade_length > 0:
+        try:
+            from kagefunc import crossfade
+        except ModuleNotFoundError:
+            raise ModuleNotFoundError("overlay_sign: 'missing dependency `kagefunc`!'")
 
     check_variable(clip, "overlay_sign")
     assert clip.format
@@ -597,10 +364,9 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
     elif isinstance(overlay, str):
         overlay = core.imwri.Read(overlay, alpha=True)
 
-    if not all([clip.width, overlay.width]) or not all([clip.height, overlay.height]):
+    if (clip.width != overlay.width) or (clip.height != overlay.height):
         raise ValueError("overlay_sign: 'Your overlay clip must have the same dimensions as your input clip!'")
 
-    # wtf mypy this IS reachable
     if isinstance(frame_ranges, list) and len(frame_ranges) > 1:
         warnings.warn("overlay_sign: 'Only one range is currently supported! "
                       "Grabbing the first item in list.'")
@@ -612,8 +378,7 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
         matrix = get_prop(clip.get_frame(0), "_Matrix", int)
 
     if matrix == 2:
-        raise ValueError("overlay_sign: 'You can't set a matrix of 2! "
-                         "Please set the correct matrix in the parameters!'")
+        raise InvalidMatrixError("overlay_sign")
 
     assert overlay.format
 
@@ -629,9 +394,9 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
         if ov_type is str:
             raise ValueError("overlay_sign: 'Please make sure your image has an alpha channel!'")
         else:
-            raise ValueError("overlay_sign: 'Please make sure you loaded your sign in using imwri.Read!'")
+            raise TypeError("overlay_sign: 'Please make sure you loaded your sign in using imwri.Read!'")
 
-    merge = core.std.MaskedMerge(clip, overlay, depth(mask, get_depth(overlay)))
+    merge = core.std.MaskedMerge(clip, overlay, depth(mask, get_depth(overlay)).std.Limiter())
 
     if not frame_ranges:
         return merge
@@ -648,8 +413,6 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
 
 
 # Aliases
-replace_ranges = _replace_ranges
-scale_thresh = _scale_thresh
 ef = edgefixer
 src = source
 
