@@ -1,31 +1,24 @@
-"""
-    Chroma reconstruction functions and wrappers.
-"""
+from __future__ import annotations
+
 from functools import partial
-from typing import List, NamedTuple
+from typing import List
 
 import vapoursynth as vs
 from vsutil import depth, get_depth, join, split
 
+from .types import RegressClips
 from .util import check_variable
 
 core = vs.core
 
 
 __all__: List[str] = [
-    'chroma_reconstruct', 'ChromaReconstruct',
-    'crecon', 'demangle',
-    'regress', 'reconstruct_multi',
-    'RegressClips'
+    'chroma_reconstruct',
+    'reconstruct_multi',
+    'regress',
 ]
 
 
-class RegressClips(NamedTuple):
-    slope: vs.VideoNode
-    intercept: vs.VideoNode
-    correlation: vs.VideoNode
-
-
 def chroma_reconstruct(clip: vs.VideoNode, radius: int = 2, i444: bool = False) -> vs.VideoNode:
     """
     A function to demangle messed-up chroma, like for example chroma
@@ -79,11 +72,11 @@ def regress(x: vs.VideoNode, *ys: vs.VideoNode, radius: int = 2, eps: float = 1e
     Fit a line for every neighborhood of values of a given size in a clip
     with corresponding neighborhoods in one or more other clips.
 
-    More info: https://en.wikipedia.org/wiki/Simple_linear_regression
+    For more info see `this Wikipedia article <https://en.wikipedia.org/wiki/Simple_linear_regression>`_.
     """
 
     if radius <= 0:
-        raise ValueError("Regress: 'radius must be greater than zero'")
+        raise ValueError("Regress: 'radius must be greater than zero!'")
 
     Expr = core.std.Expr
     E = partial(vs.core.std.BoxBlur, hradius=radius, vradius=radius)
@@ -115,6 +108,15 @@ def regress(x: vs.VideoNode, *ys: vs.VideoNode, radius: int = 2, eps: float = 1e
 
 
 def reconstruct_multi(c: vs.VideoNode, r: RegressClips, radius: int = 2) -> vs.VideoNode:
+    """
+    Tries to reconstruct regressed clips using a base video clip.
+
+    :param c:       Original clip.
+    :param r:       Regressed clips.
+    :param radius:  Internal boxblur radii.
+
+    :returns:       Fixed regressed clips.
+    """
     check_variable(c, "reconstruct_multi")
 
     weights = core.std.Expr(r.correlation, 'x 0.5 - 0.5 / 0 max')
@@ -126,9 +128,3 @@ def reconstruct_multi(c: vs.VideoNode, r: RegressClips, radius: int = 2) -> vs.V
 
 def _mean(c: vs.VideoNode, radius: int) -> vs.VideoNode:
     return core.std.BoxBlur(c, hradius=radius, vradius=radius)
-
-
-# Aliases
-ChromaReconstruct = chroma_reconstruct
-crecon = chroma_reconstruct
-demangle = chroma_reconstruct
