@@ -1,10 +1,6 @@
-"""
-    Clip rendering helpers.
-"""
 from __future__ import annotations
 
 from concurrent.futures import Future
-from enum import Enum
 from functools import partial
 from threading import Condition
 from typing import BinaryIO, Callable, Dict, List, TextIO
@@ -13,6 +9,7 @@ import vapoursynth as vs
 
 from .progress import (BarColumn, FPSColumn, Progress, TextColumn,
                        TimeRemainingColumn)
+from .types import SceneChangeMode
 from .util import get_prop
 
 core = vs.core
@@ -21,11 +18,11 @@ RenderCallback = Callable[[int, vs.VideoFrame], None]
 
 
 __all__: List[str] = [
-    'RenderContext',
-    'finish_frame',
     'clip_async_render',
+    'find_scene_changes',
+    'finish_frame',
     'get_render_progress',
-    'find_scene_changes', 'SceneChangeMode'
+    'RenderContext',
 ]
 
 
@@ -88,7 +85,7 @@ def clip_async_render(clip: vs.VideoNode,
     :param timecodes: Timecode v2 file TextIO handle. If None, timecodes will not be written.
     :param progress:  String to use for render progress display.
                       If empty or ``None``, no progress display.
-    :param callback:  Single or list of callbacks to be preformed. The callbacks are called
+    :param callback:  Single or list of callbacks to be performed. The callbacks are called
                       when each sequential frame is output, not when each frame is done.
                       Must have signature ``Callable[[int, vs.VideoNode], None]``
                       See :py:func:`lvsfunc.comparison.diff` for a use case (Default: None).
@@ -169,7 +166,7 @@ def clip_async_render(clip: vs.VideoNode,
             elif ss == (0, 1):
                 y4mformat = "440"
             else:
-                raise ValueError("clip_async_render: 'What have you done'")
+                raise ValueError("clip_async_render: 'What have you done?'")
 
         y4mformat = f"{y4mformat}p{clip.format.bits_per_sample}" if clip.format.bits_per_sample > 8 else y4mformat
 
@@ -200,6 +197,7 @@ def clip_async_render(clip: vs.VideoNode,
 
 
 def get_render_progress() -> Progress:
+    """Returns render progress."""
     return Progress(
         TextColumn("{task.description}"),
         BarColumn(),
@@ -210,13 +208,6 @@ def get_render_progress() -> Progress:
     )
 
 
-class SceneChangeMode(Enum):
-    WWXD = 0
-    SCXVID = 1
-    WWXD_SCXVID_UNION = 2
-    WWXD_SCXVID_INTERSECTION = 3
-
-
 def find_scene_changes(clip: vs.VideoNode, mode: SceneChangeMode = SceneChangeMode.WWXD) -> List[int]:
     """
     Generate a list of scene changes (keyframes).
