@@ -1,20 +1,20 @@
-"""
-    (De)scaling and conversion functions and wrappers.
-"""
 from __future__ import annotations
 
 import math
 from functools import partial
-from typing import Any, Callable, Dict, List, Literal, NamedTuple, cast
+from typing import Any, Callable, Dict, List, cast
 
 import vapoursynth as vs
 from vsutil import depth, get_depth, get_w, get_y, iterate, join, plane
 
+from .exceptions import CompareSameKernelError
 from .kernels import (Bicubic, BicubicSharp, Catrom, Kernel, Spline36,
                       get_kernel)
-from .types import VSFunction
-from .util import (check_variable, get_coefs, get_prop, quick_resample,
-                   scale_thresh)
+from .types import (CURVES, CreditMask, CustomScaler, Resolution, ScaleAttempt,
+                    VSFunction)
+from .util import (check_variable, check_variable_format,
+                   check_variable_resolution, get_coefs, get_matrix,
+                   get_matrix_curve, get_prop, quick_resample, scale_thresh)
 
 try:
     from cytoolz import functoolz
@@ -28,44 +28,18 @@ core = vs.core
 
 
 __all__: List[str] = [
-    'descale', 'descale_detail_mask', 'reupscale',
-    'ssim_downsample', 'CURVES',
-    'gamma2linear', 'linear2gamma',
-    'comparative_descale', 'comparative_restore',
-    'mixed_rescale'
+    'comparative_descale',
+    'comparative_restore',
+    'descale_detail_mask',
+    'descale',
+    'gamma2linear',
+    'linear2gamma',
+    'mixed_rescale',
+    'reupscale',
+    'ssim_downsample',
 ]
 
 
-class Resolution(NamedTuple):
-    """ Tuple representing a resolution. """
-
-    width: int
-    """ Width. """
-
-    height: int
-    """ Height. """
-
-
-class ScaleAttempt(NamedTuple):
-    """ Tuple representing a descale attempt. """
-
-    descaled: vs.VideoNode
-    """ Descaled frame in native resolution. """
-
-    rescaled: vs.VideoNode
-    """ Descaled frame reupscaled with the same kernel. """
-
-    resolution: Resolution
-    """ The native resolution. """
-
-    diff: vs.VideoNode
-    """ The subtractive difference between the original and descaled frame. """
-
-
-CreditMask = Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode]
-CustomScaler = Callable[[vs.VideoNode, int, int], vs.VideoNode]
-
-
 def _transpose_shift(n: int, f: vs.VideoFrame, clip: vs.VideoNode,
                      kernel: Kernel, caller: str) -> vs.VideoNode:
     h = f.height
@@ -91,7 +65,7 @@ def _select_descale(n: int, f: vs.VideoFrame | List[vs.VideoFrame],
         f = [f]
     f = cast(List[vs.VideoFrame], f)
     best_res = max(f, key=lambda frame:
-                   math.log(clip.height - get_prop(frame, "descaleResolution", int), 2)
+                   math.log(clip.height - get_prop(frame, "descaleResolution", int), 2)  # type:ignore[no-any-return]
                    * round(1 / max(get_prop(frame, "PlaneStatsAverage", float), 1e-12))
                    ** 0.2)
 
@@ -162,13 +136,13 @@ def descale_detail_mask(clip: vs.VideoNode, rescaled_clip: vs.VideoNode,
 
     :return:               Mask of lost detail
     """
-    check_variable(clip, "descale_detail_mask")
-    check_variable(rescaled_clip, "descale_detail_mask")
+    check_variable_resolution(clip, "descale_detail_mask")
+    check_variable_resolution(rescaled_clip, "descale_detail_mask")
 
     mask = core.std.Expr([get_y(clip), get_y(rescaled_clip)], 'x y - abs') \
         .std.Binarize(threshold)
     mask = iterate(mask, core.std.Maximum, 4)
-    return iterate(mask, core.std.Inflate, 2)
+    return iterate(mask, core.std.Inflate, 2).std.Limiter()
 
 
 def descale(clip: vs.VideoNode,
@@ -207,7 +181,7 @@ def descale(clip: vs.VideoNode,
                                     If a single height is given and upscaler is None,
                                     a constant resolution GRAY clip will be returned instead.
                                     Note that if upscaler is None, no upscaling will be performed
-                                    and neither detail masking nor proper fractional descaling can be preformed.
+                                    and neither detail masking nor proper fractional descaling can be performed.
                                     (Default: :py:func:`lvsfunc.scale.reupscale`)
     :param width:                   Width(s) to descale to (if None, auto-calculated)
     :param height:                  Height(s) to descale to. List indicates multiple resolutions,
@@ -243,12 +217,12 @@ def descale(clip: vs.VideoNode,
     width = cast(List[int], width)
 
     if len(width) != len(height):
-        raise ValueError("descale: Asymmetric number of heights and widths specified")
+        raise ValueError("descale: Asymmetric number of heights and widths specified!")
 
     resolutions = [Resolution(*r) for r in zip(width, height)]
 
     clip = depth(clip, 32)
-    assert clip.format is not None  # clip was modified by depth, but that wont make it variable
+    assert clip.format is not None  # clip was modified by depth, but that won't make it variable
     clip_y = get_y(clip) \
         .std.SetFrameProp('descaleResolution', intval=clip.height)
 
@@ -287,7 +261,7 @@ def descale(clip: vs.VideoNode,
                                        src_top=-src_top)
 
     if upscaled.format is None:
-        raise RuntimeError("descale: 'Upscaler cannot return variable-format clips'")
+        raise RuntimeError("descale: 'Upscaler cannot return variable-format clips!'")
 
     if mask:
         clip_y = clip_y.resize.Point(format=upscaled.format.id)
@@ -313,23 +287,13 @@ def descale(clip: vs.VideoNode,
     return join([upscaled, plane(clip, 1), plane(clip, 2)])
 
 
-CURVES = Literal[
-    vs.TransferCharacteristics.TRANSFER_IEC_61966_2_1,
-    vs.TransferCharacteristics.TRANSFER_BT709,
-    vs.TransferCharacteristics.TRANSFER_BT601,
-    vs.TransferCharacteristics.TRANSFER_ST240_M,
-    vs.TransferCharacteristics.TRANSFER_BT2020_10,
-    vs.TransferCharacteristics.TRANSFER_BT2020_12,
-]
-
-
 def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int = 720,
                     smooth: float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
                     kernel: Kernel | str = Catrom(), gamma: bool = False,
-                    curve: CURVES = vs.TransferCharacteristics.TRANSFER_BT709,
+                    curve: CURVES | None = None,
                     sigmoid: bool = False, epsilon: float = 1e-6) -> vs.VideoNode:
     """
-    muvsfunc.ssim_downsample rewrite taken from a Vardë gist.
+    muvsfunc.SSIM_downsample rewrite taken from a Vardë gist.
     Unlike muvsfunc's implementation, this function also works in float and does not use nnedi3_resample.
     Most of the documentation is taken from muvsfunc.
 
@@ -342,26 +306,30 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
     The downscaled images retain perceptually important features and details,
     resulting in an accurate and spatio-temporally consistent representation of the high resolution input.
 
-    Original gist: https://gist.github.com/Ichunjo/16ab1f893588aafcb096c1f35a0cfb15
+    `Original gist <https://gist.github.com/Ichunjo/16ab1f893588aafcb096c1f35a0cfb15>`_
 
-    :param clip:        Input clip
-    :param width:       Output width. If None, autocalculates using height
-    :param height:      Output height (default: 720)
+    :param clip:        Input clip.
+    :param width:       Output width. If None, autocalculates using height.
+    :param height:      Output height (default: 720).
     :param smooth:      Image smoothening method.
-                        If you pass an int, it specifies the "radius" of the internel used boxfilter,
+                        If you pass an int, it specifies the "radius" of the internally-used boxfilter,
                         i.e. the window has a size of (2*smooth+1)x(2*smooth+1).
                         If you pass a float, it specifies the "sigma" of core.tcanny.TCanny,
                         i.e. the standard deviation of gaussian blur.
                         If you pass a function, it acts as a general smoother.
                         Default uses a gaussian blur.
-    :param curve:       Gamma mapping
+    :param curve:       Gamma mapping. Will auto-determine based on the input props or resolution.
+                        Can be forced with for example `curve=vs.TransferCharacteristics.TRANSFER_BT709`.
+    :param gamma:       Perform a gamma conversion prior to scaling and after scaling.
+                        This function MUST be set to `True` for `sigmoid` and `epsilon` to function.
     :param sigmoid:     When True, applies a sigmoidal curve after the power-like curve
                         (or before when converting from linear to gamma-corrected).
                         This helps reduce the dark halo artefacts found around sharp edges
                         caused by resizing in linear luminance.
-    :param epsilon:     Machine epsilon
+                        This parameter only works if `gamma=True`.
+    :param epsilon:     Machine epsilon. This parameter only works if `gamma=True`.
 
-    :return: Downsampled clip
+    :return:            Downsampled clip.
     """
     check_variable(clip, "ssim_downsample")
 
@@ -381,6 +349,12 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
     clip = depth(clip, 32)
 
     if gamma:
+        if curve is None:
+            try:
+                curve = get_matrix_curve(get_prop(clip.get_frame(0), "_Matrix", int))
+            except ValueError:
+                curve = get_matrix_curve(get_matrix(clip, return_matrix=False))
+
         clip = gamma2linear(clip, curve, sigmoid=sigmoid, epsilon=epsilon)
 
     l1 = kernel.scale(clip, width, height)
@@ -398,19 +372,20 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
     d = core.std.Expr([m, r, l1, t], 'x y z * + a -')
 
     if gamma:
-        d = linear2gamma(d, curve, sigmoid=sigmoid)
-
+        curve = cast(CURVES, curve)
+        return linear2gamma(d, curve, sigmoid=sigmoid)
     return d
 
 
 def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  epsilon: float = 1e-6) -> vs.VideoNode:
-    check_variable(clip, "gamma2linear")
+    """Converts from gamma to linear."""
+    check_variable_format(clip, "gamma2linear")
     assert clip.format
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
-        raise ValueError("gamma2linear: 'Only 32 bits float is allowed'")
+        raise ValueError("gamma2linear: 'Your clip must be 32bit float!'")
 
     c = get_coefs(curve)
 
@@ -428,11 +403,12 @@ def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
 def linear2gamma(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  ) -> vs.VideoNode:
-    check_variable(clip, "linear2gamma")
+    """Converts from linear to gamma."""
+    check_variable_format(clip, "linear2gamma")
     assert clip.format
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
-        raise ValueError("linear2gamma: 'Only 32 bits float is allowed'")
+        raise ValueError("linear2gamma: 'Your clip must be 32bit float!'")
 
     c = get_coefs(curve)
 
@@ -471,7 +447,7 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
 
         return sharp if other_diff - thr > sharp_diff else other
 
-    check_variable(clip, "comparative_descale")
+    check_variable_resolution(clip, "comparative_descale")
 
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
@@ -480,7 +456,7 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
     kernel = kernel or Spline36()
 
     if isinstance(kernel, Bicubic) and bsharp.b == kernel.b and bsharp.c == kernel.c:
-        raise ValueError("comparative_descale: 'You may not compare BicubicSharp with itself!'")
+        raise CompareSameKernelError("comparative_descale", kernel=bsharp)
 
     if width is None:
         width = get_w(height, aspect_ratio=clip.width/clip.height)
@@ -518,7 +494,7 @@ def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: in
 
     :return:            Reupscaled clip
     """
-    check_variable(clip, "comparative_restore")
+    check_variable_resolution(clip, "comparative_restore")
 
     if isinstance(kernel, str):
         kernel = get_kernel(kernel)()
@@ -527,7 +503,7 @@ def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: in
     kernel = kernel or Spline36()
 
     if isinstance(kernel, Bicubic) and bsharp.b == kernel.b and bsharp.c == kernel.c:
-        raise ValueError("comparative_restore: 'You may not compare BicubicSharp with itself!'")
+        raise CompareSameKernelError("comparative_descale", kernel=bsharp)
 
     if width is None:
         width = get_w(height, aspect_ratio=clip.width/clip.height)
@@ -582,7 +558,6 @@ def mixed_rescale(clip: vs.VideoNode, width: None | int = None, height: int = 72
     :return:                Rescaled clip with a downscaled clip merged with it and credits masked.
     """
     check_variable(clip, "mixed_rescale")
-
     assert clip.format
 
     if not 0 <= mask_thr <= 1:
